@@ -6,29 +6,13 @@ from functools import partial
 
 import torch
 import torch.nn as nn
-import torch.nn.functional as F
 
-from .helpers import build_model_with_cfg
-from .layers import ConvBnAct, create_conv2d, create_pool2d, create_classifier
-from .registry import register_model
+from timm.layers import ConvNormAct, create_conv2d, create_pool2d, create_classifier
+from ._builder import build_model_with_cfg
+from ._registry import register_model, generate_default_cfgs
 
 __all__ = ['NASNetALarge']
 
-default_cfgs = {
-    'nasnetalarge': {
-        'url': 'http://data.lip6.fr/cadene/pretrainedmodels/nasnetalarge-a1897284.pth',
-        'input_size': (3, 331, 331),
-        'pool_size': (11, 11),
-        'crop_pct': 0.911,
-        'interpolation': 'bicubic',
-        'mean': (0.5, 0.5, 0.5),
-        'std': (0.5, 0.5, 0.5),
-        'num_classes': 1000,
-        'first_conv': 'conv0.conv',
-        'classifier': 'last_linear',
-        'label_offset': 1,  # 1001 classes in pretrained weights
-    },
-}
 
 
 class ActConvBn(nn.Module):
@@ -146,7 +130,7 @@ class CellStem1(nn.Module):
         self.path_1 = nn.Sequential()
         self.path_1.add_module('avgpool', nn.AvgPool2d(1, stride=2, count_include_pad=False))
         self.path_1.add_module('conv', nn.Conv2d(self.stem_size, self.num_channels // 2, 1, stride=1, bias=False))
-       
+
         self.path_2 = nn.Sequential()
         self.path_2.add_module('pad', nn.ZeroPad2d((-1, 1, -1, 1)))
         self.path_2.add_module('avgpool', nn.AvgPool2d(1, stride=2, count_include_pad=False))
@@ -407,20 +391,29 @@ class ReductionCell1(nn.Module):
 class NASNetALarge(nn.Module):
     """NASNetALarge (6 @ 4032) """
 
-    def __init__(self, num_classes=1000, in_chans=3, stem_size=96, channel_multiplier=2,
-                 num_features=4032, output_stride=32, drop_rate=0., global_pool='avg', pad_type='same'):
+    def __init__(
+            self,
+            num_classes=1000,
+            in_chans=3,
+            stem_size=96,
+            channel_multiplier=2,
+            num_features=4032,
+            output_stride=32,
+            drop_rate=0.,
+            global_pool='avg',
+            pad_type='same',
+    ):
         super(NASNetALarge, self).__init__()
         self.num_classes = num_classes
         self.stem_size = stem_size
-        self.num_features = num_features
+        self.num_features = self.head_hidden_size = num_features
         self.channel_multiplier = channel_multiplier
-        self.drop_rate = drop_rate
         assert output_stride == 32
 
         channels = self.num_features // 24
         # 24 is default value for the architecture
 
-        self.conv0 = ConvBnAct(
+        self.conv0 = ConvNormAct(
             in_channels=in_chans, out_channels=self.stem_size, kernel_size=3, padding=0, stride=2,
             norm_layer=partial(nn.BatchNorm2d, eps=0.001, momentum=0.1), apply_act=False)
 
@@ -500,13 +493,30 @@ class NASNetALarge(nn.Module):
             dict(num_chs=4032, reduction=32, module='act'),
         ]
 
-        self.global_pool, self.last_linear = create_classifier(
-            self.num_features, self.num_classes, pool_type=global_pool)
+        self.global_pool, self.head_drop, self.last_linear = create_classifier(
+            self.num_features, self.num_classes, pool_type=global_pool, drop_rate=drop_rate)
 
-    def get_classifier(self):
+    @torch.jit.ignore
+    def group_matcher(self, coarse=False):
+        matcher = dict(
+            stem=r'^conv0|cell_stem_[01]',
+            blocks=[
+                (r'^cell_(\d+)', None),
+                (r'^reduction_cell_0', (6,)),
+                (r'^reduction_cell_1', (12,)),
+            ]
+        )
+        return matcher
+
+    @torch.jit.ignore
+    def set_grad_checkpointing(self, enable=True):
+        assert not enable, 'gradient checkpointing not supported'
+
+    @torch.jit.ignore
+    def get_classifier(self) -> nn.Module:
         return self.last_linear
 
-    def reset_classifier(self, num_classes, global_pool='avg'):
+    def reset_classifier(self, num_classes: int, global_pool: str = 'avg'):
         self.num_classes = num_classes
         self.global_pool, self.last_linear = create_classifier(
             self.num_features, self.num_classes, pool_type=global_pool)
@@ -542,24 +552,46 @@ class NASNetALarge(nn.Module):
         x = self.act(x_cell_17)
         return x
 
+    def forward_head(self, x, pre_logits: bool = False):
+        x = self.global_pool(x)
+        x = self.head_drop(x)
+        return x if pre_logits else self.last_linear(x)
+
     def forward(self, x):
         x = self.forward_features(x)
-        x = self.global_pool(x)
-        if self.drop_rate > 0:
-            x = F.dropout(x, self.drop_rate, training=self.training)
-        x = self.last_linear(x)
+        x = self.forward_head(x)
         return x
 
 
 def _create_nasnet(variant, pretrained=False, **kwargs):
     return build_model_with_cfg(
-        NASNetALarge, variant, pretrained, default_cfg=default_cfgs[variant],
+        NASNetALarge,
+        variant,
+        pretrained,
         feature_cfg=dict(feature_cls='hook', no_rewrite=True),  # not possible to re-write this model
-        **kwargs)
+        **kwargs,
+    )
+
+
+default_cfgs = generate_default_cfgs({
+    'nasnetalarge.tf_in1k': {
+        'hf_hub_id': 'timm/',
+        'url': 'https://github.com/rwightman/pytorch-image-models/releases/download/v0.1-weights/nasnetalarge-dc4a7b8b.pth',
+        'input_size': (3, 331, 331),
+        'pool_size': (11, 11),
+        'crop_pct': 0.911,
+        'interpolation': 'bicubic',
+        'mean': (0.5, 0.5, 0.5),
+        'std': (0.5, 0.5, 0.5),
+        'num_classes': 1000,
+        'first_conv': 'conv0.conv',
+        'classifier': 'last_linear',
+    },
+})
 
 
 @register_model
-def nasnetalarge(pretrained=False, **kwargs):
+def nasnetalarge(pretrained=False, **kwargs) -> NASNetALarge:
     """NASNet-A large model architecture.
     """
     model_kwargs = dict(pad_type='same', **kwargs)
